import React, { useState, useRef, useEffect } from 'react';
import { Settings } from './components/Settings';
import { RoundsHistory } from './components/RoundsHistory';
import { Instructions } from './components/Instructions';
import { saveState, loadState, exportToCSV } from './utils/storage';
import {
  Target,
  Stats,
  ViewportOffset,
  AccelerationSettings,
  Round,
  CONSTANTS,
} from './types';
import { generateTargets, getRandomTarget } from './utils/target';
import { 
  calculateInitialViewport,
  clampZoom,
  calculateTouchDistance,
  calculateZoomFactor
} from './utils/interaction';
import { 
  createNewRound, 
  createInitialStats, 
  calculateAverageTime 
} from './utils/stats';
import { calculateAccelerationFactor, processWheelDelta, updateWheelSamples } from './utils/acceleration';
import { createDefaultSettings } from './utils/settings';
import { clamp } from './utils/math';
import './App.css';

function App() {
  // Core state
  const [targets, setTargets] = useState<Target[]>([]);
  const [currentTarget, setCurrentTarget] = useState<number | null>(null);
  const [zoomLevel, setZoomLevel] = useState<number>(CONSTANTS.ZOOM_LEVELS.default);
  const [stats, setStats] = useState<Stats>(createInitialStats());

  // Load settings from localStorage or use defaults
  const [settings, setSettings] = useState<AccelerationSettings>(() => {
    const savedState = loadState();
    const defaultSettings = createDefaultSettings();
    return savedState?.settings ?? defaultSettings;
  });

  const [rounds, setRounds] = useState<Round[]>(() => {
    const savedState = loadState();
    return savedState?.rounds ?? [];
  });

  // UI state
  const [showSettings, setShowSettings] = useState(false);
  const [showRounds, setShowRounds] = useState(false);
  const [showInstructions, setShowInstructions] = useState(false);
  const [viewportOffset, setViewportOffset] = useState<ViewportOffset>({
    x: 0,
    y: 0,
  });

  // Add state to track drag operations
  const isDragging = useRef(false);
  const dragStartTime = useRef(0);
  const dragStartPos = useRef({ x: 0, y: 0 });
  const DRAG_THRESHOLD = 5; // pixels

  // Refs for handling zoom behavior
  const containerRef = useRef<HTMLDivElement>(null);
  const zoomSpeedSamples = useRef<number[]>([]);
  const touchStartDistance = useRef<number>(0);
  const wasReset = useRef(false);

  useEffect(() => {
    const savedState = loadState();
    if (savedState?.stats) {
      setStats(savedState.stats);
    }
    setShowInstructions(!savedState?.rounds || savedState.rounds.length === 0);
  }, []);

  useEffect(() => {
    saveState({
      rounds,
      stats,
      settings,
    });
  }, [rounds, stats, settings]);

  useEffect(() => {
    initializeGame();
  }, []);

  const initializeGame = () => {
    const newTargets = generateTargets();
    setTargets(newTargets);
    setCurrentTarget(getRandomTarget(null, CONSTANTS.TARGET_COUNT));

    const initialZoom = 0.2;
    setZoomLevel(initialZoom);
    setViewportOffset(calculateInitialViewport(initialZoom));
    setStats(previous => ({ ...previous, startTime: Date.now() }));
  };

  useEffect(() => {
    const container = containerRef.current;
    if (!container) return;

    const handleWheelEvent = (event: WheelEvent) => {
      if (event.ctrlKey || event.metaKey) {
        event.preventDefault();
    
        const containerBounds = container.getBoundingClientRect();
        const cursorX = event.clientX - containerBounds.left;
        const cursorY = event.clientY - containerBounds.top;

        const worldX = viewportOffset.x + cursorX / zoomLevel;
        const worldY = viewportOffset.y + cursorY / zoomLevel;

        const scrollAmount = processWheelDelta(-event.deltaY);
        
        if (isFinite(scrollAmount)) {
          zoomSpeedSamples.current = updateWheelSamples(zoomSpeedSamples.current, scrollAmount);
        }

        let zoomDelta = scrollAmount * 0.001;

        if (settings.enabled && zoomSpeedSamples.current.length > 0) {
          const accelerationFactor = calculateAccelerationFactor(
            zoomSpeedSamples.current,
            settings.accelerationCurve,
            settings.enabled
          );
          
          if (isFinite(accelerationFactor)) {
            zoomDelta *= accelerationFactor;
          }
        }

        zoomDelta = clamp(zoomDelta, -0.5, 0.5);
        const newZoom = clampZoom(zoomLevel * (1 + zoomDelta));

        if (isFinite(newZoom) && newZoom > 0) {
          const newOffsetX = worldX - cursorX / newZoom;
          const newOffsetY = worldY - cursorY / newZoom;

          if (isFinite(newOffsetX) && isFinite(newOffsetY)) {
            setZoomLevel(newZoom);
            setViewportOffset({ x: newOffsetX, y: newOffsetY });
          }
        }
      }
    };

    container.addEventListener('wheel', handleWheelEvent, { passive: false });
    return () => container.removeEventListener('wheel', handleWheelEvent);
  }, [zoomLevel, viewportOffset, settings]);

  const handleTouchStart = (event: React.TouchEvent<HTMLDivElement>) => {
    if (event.touches.length === 2) {
      const touch1 = event.touches[0];
      const touch2 = event.touches[1];
      touchStartDistance.current = calculateTouchDistance(touch1, touch2);
    }
  };

  const handleTouchMove = (event: React.TouchEvent<HTMLDivElement>) => {
    if (event.touches.length === 2) {
      event.preventDefault();
      const touch1 = event.touches[0];
      const touch2 = event.touches[1];
      const currentDistance = calculateTouchDistance(touch1, touch2);
      const zoomFactor = calculateZoomFactor(currentDistance, touchStartDistance.current);
      setZoomLevel(previous => clampZoom(previous * zoomFactor));
    }
  };

  const handleTouchEnd = () => {
    touchStartDistance.current = 0;
  };

  const handleMouseMove = (event: React.MouseEvent) => {
    if (event.buttons === 1) {
      // Calculate total drag distance
      const dx = event.clientX - dragStartPos.current.x;
      const dy = event.clientY - dragStartPos.current.y;
      const dragDistance = Math.hypot(dx, dy);

      // Only set dragging if we've moved past the threshold
      if (dragDistance > DRAG_THRESHOLD) {
        isDragging.current = true;
      }

      setViewportOffset(previous => ({
        x: previous.x - event.movementX / zoomLevel,
        y: previous.y - event.movementY / zoomLevel,
      }));
    }
  };

  const handleMouseDown = (event: React.MouseEvent) => {
    dragStartTime.current = Date.now();
    dragStartPos.current = { x: event.clientX, y: event.clientY };
    isDragging.current = false;
  };

  const handleMouseUp = () => {
    // Reset drag state after a short delay to allow click event to check it
    setTimeout(() => {
      isDragging.current = false;
      dragStartPos.current = { x: 0, y: 0 };
    }, 0);
  };

  const handleClick = (event: React.MouseEvent) => {
    if (!containerRef.current || currentTarget === null) return;

    // If this was a drag operation or very slow click (likely a drag attempt), ignore it
    if (isDragging.current || Date.now() - dragStartTime.current > 200) {
      return;
    }

    const containerBounds = containerRef.current.getBoundingClientRect();
    const worldX = (event.clientX - containerBounds.left) / zoomLevel + viewportOffset.x;
    const worldY = (event.clientY - containerBounds.top) / zoomLevel + viewportOffset.y;

    const target = targets[currentTarget];
    if (!target) return;

    const distanceToTarget = Math.hypot(target.x - worldX, target.y - worldY);

    if (distanceToTarget <= target.radius) {
      const timeElapsed = Date.now() - stats.startTime;
      setRounds(previous => [...previous, createNewRound(timeElapsed, stats.misclicks, settings)]);
      setStats({
        startTime: Date.now(),
        misclicks: 0,
        targetsHit: stats.targetsHit + 1,
        times: [...stats.times, timeElapsed],
      });
      setCurrentTarget(getRandomTarget(currentTarget, CONSTANTS.TARGET_COUNT));
    } else {
      setStats(previous => ({
        ...previous,
        misclicks: previous.misclicks + 1,
      }));
    }
  };

  const handleReset = () => {
    if (window.confirm('Are you sure you want to reset all trials, history, and settings? This cannot be undone.')) {
      setRounds([]);
      setStats(createInitialStats());
      setSettings(createDefaultSettings());
      initializeGame();
      wasReset.current = true;
      setShowInstructions(true);
    }
  };

  return (
    <div className="app">
      <div className="controls">
        <button 
          className="settings-toggle" 
          onClick={() => setShowSettings(!showSettings)}
        >
          ⚙️ Settings
        </button>
        
        <button 
          className="rounds-toggle" 
          onClick={() => setShowRounds(!showRounds)}
        >
          📊 History
        </button>

        <button 
          className="export-button" 
          onClick={() => exportToCSV(rounds)}
          disabled={rounds.length === 0}
        >
          📥 Export CSV
        </button>

        <button 
          className="reset-button" 
          onClick={handleReset}
        >
          🔄 Reset
        </button>
      </div>
      
      <Settings
        settings={settings}
        onSettingsChange={setSettings}
        visible={showSettings}
        onClose={() => setShowSettings(false)}
      />
      
      <RoundsHistory
        rounds={rounds}
        visible={showRounds}
        onClose={() => setShowRounds(false)}
      />

      <Instructions
        visible={showInstructions}
        onClose={() => setShowInstructions(false)}
      />
      
      <div className="stats">
        <span>Targets: {stats.targetsHit}</span> • 
        <span>Misclicks: {stats.misclicks}</span> • 
        <span>Avg Time: {calculateAverageTime(stats.times)} ms</span> • 
        <span>Zoom: {zoomLevel.toFixed(2)}x</span>
      </div>

      <div
        ref={containerRef}
        className="container"
        onTouchStart={handleTouchStart}
        onTouchMove={handleTouchMove}
        onTouchEnd={handleTouchEnd}
        onClick={handleClick}
        onMouseMove={handleMouseMove}
        onMouseDown={handleMouseDown}
        onMouseUp={handleMouseUp}
      >
        <div
          className="content"
          style={{
            transform: `translate(${-viewportOffset.x * zoomLevel}px, ${-viewportOffset.y * zoomLevel}px) scale(${zoomLevel})`,
            transformOrigin: '0 0',
          }}
        >
          {targets.map((target, index) => (
            <div
              key={index}
              className={`target ${index === currentTarget ? 'active' : ''}`}
              style={{
                left: `${target.x}px`,
                top: `${target.y}px`,
                width: `${target.radius * 2}px`,
                height: `${target.radius * 2}px`,
              }}
            />
          ))}
        </div>
      </div>
    </div>
  );
}

export default App;
